<?php

namespace Drupal\navigation_blocks;

use Drupal\Component\Utility\Unicode;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Path\AliasManagerInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class BlockPathMatcher
 *
 * @package Drupal\navigation_blocks
 */
class PathMatcher implements ContainerInjectionInterface {

  /**
   * Current Path.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * Path Matcher.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * Alias Manager.
   *
   * @var \Drupal\Core\Path\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * Constructs a new BlockPathMatcher object.
   *
   * @param \Drupal\Core\Path\CurrentPathStack $currentPath
   * @param \Drupal\Core\Path\PathMatcherInterface $pathMatcher
   * @param \Drupal\Core\Path\AliasManagerInterface $aliasManager
   */
  public function __construct(CurrentPathStack $currentPath, PathMatcherInterface $pathMatcher, AliasManagerInterface $aliasManager) {
    $this->currentPath = $currentPath;
    $this->pathMatcher = $pathMatcher;
    $this->aliasManager = $aliasManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('path.current'),
      $container->get('path.matcher'),
      $container->get('path.alias_manager')
    );
  }

  /**
   * Validate if the current path is valid.
   *
   * @param \Drupal\Core\Url $url
   *   Url
   *
   * @return bool
   *   TRUE if it validates.
   */
  public function validateCurrentPath(Url $url) {
    $current_path = $this->currentPath->getPath();
    return !($this->aliasManager->getAliasByPath($current_path) === '/' . $url->getInternalPath());
  }

  /**
   * Match path to a string of preferred paths for the back button.
   *
   * @param string $path
   *   Path to match with.
   * @param string $preferredPaths
   *   Preferred paths to navigate back to.
   *
   * @return bool
   */
  public function matchPath($path, $preferredPaths) {
    $path = $path === '/' ? $path : rtrim($path, '/');
    $path_alias = Unicode::strtolower($this->aliasManager->getAliasByPath($path));
    return $this->pathMatcher->matchPath($path_alias, $preferredPaths) || (($path != $path_alias) && $this->pathMatcher->matchPath($path, $preferredPaths));
  }

}
